/* DC9 update
 Esto es requerido por Dynamic C versin 9, para utilizar
una IP fija en programa y la interfaz Ethernet por defecto.
No es necesario en DC8, se autodefinen en las libs */
#define TCPCONFIG 0
#define USE_ETHERNET		1
/* Fin DC9 update */

/* TCP/IP stuff */

#define MY_IP_ADDRESS   "192.168.1.52"
#define MY_NETMASK      "255.255.255.0"

#define MY_GATEWAY		"192.168.1.1" 		// only needed if out of local LAN

#memmap xmem
#use "dcrtcp.lib"
#use "http.lib"


/* use flash import facility */

#ximport "bb.shtml"      index_html
#ximport "ok.html"      ok_html
#ximport "rabbit1.gif"    rabbit1_gif


/* the default for / must be first */
const static HttpType http_types[] =
{
   { ".shtml", "text/html", shtml_handler}, // ssi
   { ".html", "text/html", NULL},           // html
   { ".cgi", "", NULL},                     // cgi
   { ".gif", "image/gif", NULL}
};

int msg;								// count number of messages
char date[11],time[9];			// store current date&time

int submit(HttpState*);

const static HttpSpec http_flashspec[] =
{
   { HTTPSPEC_FILE,  "/",              index_html,    NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/index.shtml",   index_html,    NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/ok.html",   		ok_html,		   NULL, 0, NULL, NULL},
   { HTTPSPEC_FILE,  "/rabbit1.gif",   rabbit1_gif,   NULL, 0, NULL, NULL},

   { HTTPSPEC_VARIABLE, "msg",  0,    &msg, INT16,   "%d", NULL},
   { HTTPSPEC_VARIABLE, "date", 0,    date, PTR16,   "%s", NULL},
   { HTTPSPEC_VARIABLE, "time", 0,    time, PTR16,   "%s", NULL},

   { HTTPSPEC_FUNCTION,	"/submit.cgi",		 0,  submit, 0, NULL, NULL}
};

#define REDIRECTHOST		MY_IP_ADDRESS
#define REDIRECTTO      "http://" REDIRECTHOST "/ok.html"


/*	Hardware definitions for graphic LCD displays on RCM2100

		PA.0-7	I/O	LCD D0-7

		PE.0	Out	LCD E
		PE.4	Out	LCD ID
		PE.3	Out	LCD R/W_

		PE.1	Out	LCD CS1
		PE.7	Out	LCD CS2
*/

/* LCD control signals */
#define	LCD_CS1	1
#define	LCD_CS2	7
#define	LCD_E		0
#define	LCD_ID	4
#define	LCD_RW	3

/* LCD status bits */
#define BUSY	7


void MsDelay ( int iDelay )
{
	unsigned long ul0;
	ul0 = MS_TIMER;		// get current timer value
	while ( MS_TIMER < ul0 + (unsigned long) iDelay );
}


/* Low level functions */

void LCD_SelSide(int side)
{
	if(side)	{
		BitWrPortI ( PEDR, &PEDRShadow, 1,LCD_CS2 );	// Rise CS2
		BitWrPortI ( PEDR, &PEDRShadow, 0,LCD_CS1 );	// Lower CS1
	}
	else	{
		BitWrPortI ( PEDR, &PEDRShadow, 1,LCD_CS1 );	// Rise CS1
		BitWrPortI ( PEDR, &PEDRShadow, 0,LCD_CS2 );	// Lower CS2
	}
}

void LCD_CalcPage(int y,int *page,int *row)
{
	*page=(y>>3);				// page = integer part of y coord /8
	*row=y-((*page)<<3);		// row = remainder
}

void LCD_Write(int data)
{
	WrPortI ( PADR, &PADRShadow, data );		// write data
	WrPortI ( SPCR, &SPCRShadow, 0x84 );		// PA0-7 = Outputs, put data in bus
	BitWrPortI ( PEDR, &PEDRShadow, 0,LCD_RW );	// Lower RW (Write)
	BitWrPortI ( PEDR, &PEDRShadow, 1,LCD_E );	// Rise E
	BitWrPortI ( PEDR, &PEDRShadow, 0,LCD_E );	// Lower E
}

int LCD_Read()
{
int data;
	BitWrPortI ( PEDR, &PEDRShadow, 1,LCD_RW );	// Rise RW (Read)
	WrPortI ( SPCR, &SPCRShadow, 0x80 );		// PA0-7 = Inputs, remove data from bus
	BitWrPortI ( PEDR, &PEDRShadow, 1,LCD_E ); // Rise E
	data=RdPortI ( PADR );		   // read data
	BitWrPortI ( PEDR, &PEDRShadow, 0,LCD_E ); // Lower E
	return(data);
}

int LCD_Status()
{
int status;
	BitWrPortI ( PEDR, &PEDRShadow, 0,LCD_ID );	// Lower ID (Cmd)
	do {
		status=LCD_Read();		   // read status (busy flag)
	} while(status&(1<<BUSY));		// loop while busy=1
	return(status);
}

int LCD_ReadData()
{
	BitWrPortI ( PEDR, &PEDRShadow, 1,LCD_ID );	// Rise ID (Data)
	return(LCD_Read());
}

void LCD_WriteCmd(int cmd)
{
	LCD_Status();		// ID returns low
	LCD_Write(cmd);
}

void LCD_WriteData(int data)
{
	LCD_Status();
	BitWrPortI ( PEDR, &PEDRShadow, 1,LCD_ID );	// Rise ID (Data)
	LCD_Write(data);
}

void LCD_init ()
{
	WrPortI ( PEDR,&PEDRShadow,0x00 );				// Make sure Outputs are Low
	BitWrPortI ( PEDDR,&PEDDRShadow,1,LCD_CS1);	// PE.LCD_CS1 = output
	BitWrPortI ( PEDDR,&PEDDRShadow,1,LCD_CS2);	// PE.LCD_CS2 = output
	BitWrPortI ( PEDDR,&PEDDRShadow,1,LCD_E);		// PE.LCD_E = output
	BitWrPortI ( PEDDR,&PEDDRShadow,1,LCD_ID);	// PE.LCD_ID = output
																// PB.LCD_RW = output (PB has two outputs)
	BitWrPortI ( PEDDR,&PEDDRShadow,1,LCD_RW);	// PE.LCD_RW = output
	BitWrPortI ( PEFR, &PEFRShadow, 0,LCD_CS1 );	// PE.LCD_CS1: no I/O strobe
	BitWrPortI ( PEFR, &PEFRShadow, 0,LCD_CS2 );	// PE.LCD_CS2: no I/O strobe
	BitWrPortI ( PEFR, &PEFRShadow, 0,LCD_E );	// PE.LCD_E: no I/O strobe
	BitWrPortI ( PEFR, &PEFRShadow, 0,LCD_ID );	// PE.LCD_ID: no I/O strobe
	BitWrPortI ( PEFR, &PEFRShadow, 0,LCD_RW );	// PE.LCD_RW: no I/O strobe
	MsDelay ( 1000 );										// wait for LCD to reset itself

	LCD_SelSide(1);
	LCD_WriteCmd ( '\B00111111' );					// Display on
	LCD_SelSide(0);
	LCD_WriteCmd ( '\B00111111' );					// Display on
}

/* High level functions */

void LCD_home ( void )
{
	LCD_SelSide(1);
	LCD_WriteCmd ('\B01000000');			// address 0
	LCD_WriteCmd ('\B11000000');			// display starts at address 0
	LCD_SelSide(0);
	LCD_WriteCmd ('\B01000000');			// address 0
	LCD_WriteCmd ('\B11000000');			// display starts at address 0
}

void LCD_fill(unsigned char pattern)
{
int i,page;
	LCD_home();										// address 0
	for(page=0;page<8;page++) {
		LCD_SelSide(0);							// select left
		LCD_WriteCmd (0xB8+page);				// set page address
		for(i=0;i<64;i++)
			LCD_WriteData(pattern);				// fill with pattern
		LCD_SelSide(1);							// select right
		LCD_WriteCmd (0xB8+page);				// set page address
		for(i=0;i<64;i++)
			LCD_WriteData(pattern);				// fill with pattern
	}
}


#define LCD_clear() LCD_fill(0)


void LCD_scroll1()
{
int i,page,data;

	for(page=2;page<8;page++) {				// leave page 0 alone (datetime)
		for(i=0;i<64;i++){
			LCD_WriteCmd (0xB8+page);			// set page address
			LCD_ReadData();						// dummy read before reading
			data=LCD_ReadData();					// get data for each page
			if(page==7){
				LCD_WriteCmd (0x40+i);			// set address again (reading increments counter)
				LCD_WriteData(0);					// clear last row (page 7)
			}
			LCD_WriteCmd (0x40+i);				// set address again (reading increments counter)
			LCD_WriteCmd (0xB7+page);			// set page address at prior page
			LCD_WriteData(data);					// and move it
		}
	}
}
void LCD_scroll()
{
	LCD_home();										// address 0,0
	LCD_SelSide(1);								// select right
	LCD_scroll1();									// scroll
	LCD_SelSide(0);								// select left
	LCD_scroll1();									// scroll
}


void LCD_putchr ( char chr )
{
const static char font5x7[] = {
0x00, 0x00, 0x00, 0x00, 0x00,// (space)
0x00, 0x00, 0x5F, 0x00, 0x00,// !
0x00, 0x07, 0x00, 0x07, 0x00,// "
0x14, 0x7F, 0x14, 0x7F, 0x14,// #
0x24, 0x2A, 0x7F, 0x2A, 0x12,// $
0x23, 0x13, 0x08, 0x64, 0x62,// %
0x36, 0x49, 0x55, 0x22, 0x50,// &
0x00, 0x05, 0x03, 0x00, 0x00,// '
0x00, 0x1C, 0x22, 0x41, 0x00,// (
0x00, 0x41, 0x22, 0x1C, 0x00,// )
0x08, 0x2A, 0x1C, 0x2A, 0x08,// *
0x08, 0x08, 0x3E, 0x08, 0x08,// +
0x00, 0x50, 0x30, 0x00, 0x00,// ,
0x08, 0x08, 0x08, 0x08, 0x08,// -
0x00, 0x30, 0x30, 0x00, 0x00,// .
0x20, 0x10, 0x08, 0x04, 0x02,// /
0x3E, 0x51, 0x49, 0x45, 0x3E,// 0
0x00, 0x42, 0x7F, 0x40, 0x00,// 1
0x42, 0x61, 0x51, 0x49, 0x46,// 2
0x21, 0x41, 0x45, 0x4B, 0x31,// 3
0x18, 0x14, 0x12, 0x7F, 0x10,// 4
0x27, 0x45, 0x45, 0x45, 0x39,// 5
0x3C, 0x4A, 0x49, 0x49, 0x30,// 6
0x01, 0x71, 0x09, 0x05, 0x03,// 7
0x36, 0x49, 0x49, 0x49, 0x36,// 8
0x06, 0x49, 0x49, 0x29, 0x1E,// 9
0x00, 0x36, 0x36, 0x00, 0x00,// :
0x00, 0x56, 0x36, 0x00, 0x00,// ;
0x00, 0x08, 0x14, 0x22, 0x41,// <
0x14, 0x14, 0x14, 0x14, 0x14,// =
0x41, 0x22, 0x14, 0x08, 0x00,// >
0x02, 0x01, 0x51, 0x09, 0x06,// ?
0x32, 0x49, 0x79, 0x41, 0x3E,// @
0x7E, 0x11, 0x11, 0x11, 0x7E,// A
0x7F, 0x49, 0x49, 0x49, 0x36,// B
0x3E, 0x41, 0x41, 0x41, 0x22,// C
0x7F, 0x41, 0x41, 0x22, 0x1C,// D
0x7F, 0x49, 0x49, 0x49, 0x41,// E
0x7F, 0x09, 0x09, 0x01, 0x01,// F
0x3E, 0x41, 0x41, 0x51, 0x32,// G
0x7F, 0x08, 0x08, 0x08, 0x7F,// H
0x00, 0x41, 0x7F, 0x41, 0x00,// I
0x20, 0x40, 0x41, 0x3F, 0x01,// J
0x7F, 0x08, 0x14, 0x22, 0x41,// K
0x7F, 0x40, 0x40, 0x40, 0x40,// L
0x7F, 0x02, 0x04, 0x02, 0x7F,// M
0x7F, 0x04, 0x08, 0x10, 0x7F,// N
0x3E, 0x41, 0x41, 0x41, 0x3E,// O
0x7F, 0x09, 0x09, 0x09, 0x06,// P
0x3E, 0x41, 0x51, 0x21, 0x5E,// Q
0x7F, 0x09, 0x19, 0x29, 0x46,// R
0x46, 0x49, 0x49, 0x49, 0x31,// S
0x01, 0x01, 0x7F, 0x01, 0x01,// T
0x3F, 0x40, 0x40, 0x40, 0x3F,// U
0x1F, 0x20, 0x40, 0x20, 0x1F,// V
0x7F, 0x20, 0x18, 0x20, 0x7F,// W
0x63, 0x14, 0x08, 0x14, 0x63,// X
0x03, 0x04, 0x78, 0x04, 0x03,// Y
0x61, 0x51, 0x49, 0x45, 0x43,// Z
0x00, 0x00, 0x7F, 0x41, 0x41,// [
0x02, 0x04, 0x08, 0x10, 0x20,// "\"
0x41, 0x41, 0x7F, 0x00, 0x00,// ]
0x04, 0x02, 0x01, 0x02, 0x04,// ^
0x40, 0x40, 0x40, 0x40, 0x40,// _
0x00, 0x01, 0x02, 0x04, 0x00,// `
0x20, 0x54, 0x54, 0x54, 0x78,// a
0x7F, 0x48, 0x44, 0x44, 0x38,// b
0x38, 0x44, 0x44, 0x44, 0x20,// c
0x38, 0x44, 0x44, 0x48, 0x7F,// d
0x38, 0x54, 0x54, 0x54, 0x18,// e
0x08, 0x7E, 0x09, 0x01, 0x02,// f
0x08, 0x14, 0x54, 0x54, 0x3C,// g
0x7F, 0x08, 0x04, 0x04, 0x78,// h
0x00, 0x44, 0x7D, 0x40, 0x00,// i
0x20, 0x40, 0x44, 0x3D, 0x00,// j
0x00, 0x7F, 0x10, 0x28, 0x44,// k
0x00, 0x41, 0x7F, 0x40, 0x00,// l
0x7C, 0x04, 0x18, 0x04, 0x78,// m
0x7C, 0x08, 0x04, 0x04, 0x78,// n
0x38, 0x44, 0x44, 0x44, 0x38,// o
0x7C, 0x14, 0x14, 0x14, 0x08,// p
0x08, 0x14, 0x14, 0x18, 0x7C,// q
0x7C, 0x08, 0x04, 0x04, 0x08,// r
0x48, 0x54, 0x54, 0x54, 0x20,// s
0x04, 0x3F, 0x44, 0x40, 0x20,// t
0x3C, 0x40, 0x40, 0x20, 0x7C,// u
0x1C, 0x20, 0x40, 0x20, 0x1C,// v
0x3C, 0x40, 0x30, 0x40, 0x3C,// w
0x44, 0x28, 0x10, 0x28, 0x44,// x
0x0C, 0x50, 0x50, 0x50, 0x3C,// y
0x44, 0x64, 0x54, 0x4C, 0x44,// z
0x00, 0x08, 0x36, 0x41, 0x00,// {
0x00, 0x00, 0x7F, 0x00, 0x00,// |
0x00, 0x41, 0x36, 0x08, 0x00,// }
0x08, 0x08, 0x2A, 0x1C, 0x08,// ->
0x08, 0x1C, 0x2A, 0x08, 0x08 // <-
};
int i;
		for(i=0;i<5;i++)
			LCD_WriteData(font5x7[5*(chr-0x20)+i]);	// send character
		LCD_WriteData(0);										// send intercharacter separator
}

void LCD_printat (unsigned int row, unsigned int col, char *ptr)
{
int x;
	do {
		x=(col>9)?6*(col-10):6*col;				// correct x address if col>9
		if(col>9)										// 0-9 => side 0; 10>20 => side 1
			LCD_SelSide(1);
		else {
			LCD_SelSide(0);
			x+=4;											// 4 pixels offset at both sides
		}
		LCD_WriteCmd (0xB8+row);					// set page address
		LCD_WriteCmd (0x40+x);						// address = 6*col or 6*col-10 (4 pixels offset)
		LCD_putchr (*ptr++);
		col++;
	} 	while (*ptr);
}

/* RTC functions */

void get_datetime()
{
struct tm thetm;

	mktm(&thetm, SEC_TIMER);
	sprintf(date,"%02d/%02d/%04d",thetm.tm_mday,thetm.tm_mon,1900+thetm.tm_year);
	sprintf(time,"%02d:%02d:%02d",thetm.tm_hour, thetm.tm_min, thetm.tm_sec);
}

#define MAX_FORMSIZE	64
typedef struct {
	char *name;
	char value[MAX_FORMSIZE];
} FORMType;
FORMType FORMSpec[1];

/*
 * Parse one token 'foo=bar', matching 'foo' to the name field in
 * the struct, and storing 'bar' into the value
 */
void parse_token(HttpState* state)
{
	auto int i, len;

	for(i=0; i<HTTP_MAXBUFFER; i++) {
		if(state->buffer[i] == '=')
			state->buffer[i] = '\0';
	}
	state->p = state->buffer + strlen(state->buffer) + 1;

	for(i=0; i<(sizeof(FORMSpec)/sizeof(FORMType)); i++) {
		if(!strcmp(FORMSpec[i].name,state->buffer)) {
			len = (strlen(state->p)>MAX_FORMSIZE) ? MAX_FORMSIZE - 1: strlen(state->p);
			strncpy(FORMSpec[i].value,state->p,1+len);
			FORMSpec[i].value[MAX_FORMSIZE - 1] = '\0';
		}
	}
}

/*
 * parse the url-encoded POST data into the FORMSpec struct
 * (ie: parse 'foo=bar&baz=qux' into the struct
 */
int parse_post(HttpState* state)
{
	auto int retval;

	while(1) {
		retval = sock_fastread(&state->s, state->p, 1);
		if(0 == retval) {
			*state->p = '\0';
			parse_token(state);

			return 1;
		}

		/* should this only be '&'? (allow the eoln as valid text?) */
		if((*state->p == '&') || (*state->p == '\r') || (*state->p == '\n')) {
			/* found one token */
			*state->p = '\0';
			parse_token(state);

			state->p = state->buffer;
		} else {
			state->p++;
		}

		if((state->p - state->buffer) > HTTP_MAXBUFFER) {
			/* input too long */
			return 1;
		}
	}

	return 0; /* end of data - loop again to give it time to write more */
}


int submit(HttpState* state)
{
char buffer[MAX_FORMSIZE];

	FORMSpec[0].value[0] = 0;									// init form data
	if(parse_post(state)) {
		if(*(http_urldecode(buffer,FORMSpec[0].value,MAX_FORMSIZE))) { // decode data, ignore empty strings
			buffer[21]=0;											// cut string to display limits
			LCD_scroll();											// move up past messages
			LCD_printat(7,0,buffer);							// show current msg
			msg++;													// update msg counter
		}
		cgi_redirectto(state,REDIRECTTO);					// Show OK screen
	} else {
									// insert error handling here
	}
	return(0);
}

/* MAIN PROGRAM */

main()
{
int i;

	msg=1;
	FORMSpec[0].name = "texto";



   sock_init();
   http_init();

	LCD_init();
	LCD_clear();

	while (1) {										// endless loop

		costate {
	      http_handler();						// TCP/IP stuff
			yield;									// pass execution on to next costatement
		}

		costate {
			waitfor(DelaySec(1));				// waits until 1 second have passed
			get_datetime();						// update date & time strings
			LCD_printat(0,0,date);
			LCD_printat(0,12,time);
		}
	}
}